Package: AquaEnv
Version: 1.0-5
Title: Integrated Development Toolbox for Aquatic Chemical Model
        Generation
Authors@R: c(person(given = c("Andreas", "F."),
                        family = "Hofmann",
                        role = "aut"),
                 person(given = "Karline",
                        family = "Soetaert",
                        role = c("aut", "cre"),
                        email = "karline.soetaert@nioz.nl"),
                 person(given = c("Filip", "J.R."),
                        family = "Meysman",
                        role = "aut"),
                 person(given = "Mathilde",
                        family = "Hagens",
                        role = "aut"))
Depends: R (>= 2.15.0)
Imports: minpack.lm, graphics, grDevices, stats
Suggests: deSolve
Description: Toolbox for the experimental aquatic chemist, focused on 
        acidification and CO2 air-water exchange. It contains all elements to
        model the pH, the related CO2 air-water exchange, and
        aquatic acid-base chemistry for an arbitrary marine,
        estuarine or freshwater system. It contains a suite of tools for 
        sensitivity analysis, visualisation, modelling of chemical batches, 
        and can be used to build dynamic models of aquatic systems. 
        As from version 1.0-4, it also contains functions to calculate 
        the buffer factors. 
License: GPL (>= 2)
LazyData: yes
Repository: CRAN
NeedsCompilation: no
Packaged: 2026-02-12 17:51:37 UTC; karlines
Author: Andreas F. Hofmann [aut],
  Karline Soetaert [aut, cre],
  Filip J.R. Meysman [aut],
  Mathilde Hagens [aut]
Maintainer: Karline Soetaert <karline.soetaert@nioz.nl>
Date/Publication: 2026-02-13 07:01:17 UTC
Built: R 4.6.0; ; 2026-04-25 16:29:37 UTC; unix
