Package: AquaEnv
Version: 1.0-4
Title: Integrated Development Toolbox for Aquatic Chemical Model
        Generation
Author: Andreas F. Hofmann, Karline Soetaert, Filip J.R. Meysman, 
        Mathilde Hagens
Maintainer: Karline Soetaert <Karline.Soetaert@nioz.nl>
Depends: R (>= 2.15.0)
Imports: minpack.lm, graphics, grDevices, stats
Suggests: deSolve
Description: Toolbox for the experimental aquatic chemist, focused on 
        acidification and CO2 air-water exchange. It contains all elements to
        model the pH, the related CO2 air-water exchange, and
        aquatic acid-base chemistry for an arbitrary marine,
        estuarine or freshwater system. It contains a suite of tools for 
        sensitivity analysis, visualisation, modelling of chemical batches, 
        and can be used to build dynamic models of aquatic systems. 
        As from version 1.0-4, it also contains functions to calculate 
        the buffer factors. 
License: GPL (>= 2)
LazyData: yes
Repository: CRAN
Date/Publication: 2016-09-06 16:39:52
NeedsCompilation: no
Repository/R-Forge/Project: aquaenv
Repository/R-Forge/Revision: 97
Repository/R-Forge/DateTimeStamp: 2016-09-05 13:48:52
Packaged: 2016-09-05 14:07:37 UTC; rforge
Built: R 4.3.3; ; 2025-01-24 03:59:24 UTC; unix
