import(grDevices, stats)
importFrom(stats, approx)
importFrom(graphics, plot, par, legend, polygon)
importFrom(minpack.lm, nls.lm, nls.lm.control)
export(aquaenv, convert, c.aquaenv, TAfit, titration, K_BOH3, K_CO2, K_H2PO4, K_H2S, K_H3PO4, K_HCO3, K_HF, K_HPO4, K_HSO4, K_NH4, K_SiOH4, K_SiOOH3, K_W, K0_CO2, K0_O2, Ksp_aragonite, Ksp_calcite, PhysChemConst, Technicals, DeltaPcoeffs, MeanMolecularMass, ConcRelCl, gauge_p, watdepth, BufferFactors)

S3method("plot", "aquaenv")
S3method("as.data.frame", "aquaenv")
S3method("merge", "aquaenv")
S3method("c", "aquaenv")
S3method("length", "aquaenv")
